"""
$Id: $
"""

from zope import interface, schema, component
from getpaid.core import interfaces
from getpaid.core.interfaces import IAddress
from zope.component.interfaces import IObjectEvent
from zope.app.container.interfaces import IContainer
from zope.viewlet.interfaces import IViewletManager
from zope.schema.interfaces import IContextSourceBinder
from zope.schema import vocabulary

from i18n import _

class IWarehouse( interface.Interface ):
    name = schema.TextLine( title=_(u"Warehouse Name") )
    location = schema.Object( interfaces.IAddress )

class IWarehouseContainer( IContainer ):
    """
    warehouse container utility
    """

def WarehouseSource( *args ):
    container = component.getUtility( IWarehouseContainer )
    return vocabulary.SimpleVocabulary.fromValues( container.keys() )

interface.directlyProvides( WarehouseSource, IContextSourceBinder )

class IProductInventory( interface.Interface ):   
    pickbin = schema.TextLine( title=_(u"Pick Bin"), description=_("") )
    pallet = schema.TextLine( title=_(u"Pallet"), description=_(""))
    stock  = schema.Int( title=_(u"Quantity in Stock"), default=0)
    
    # so the notion is that we automatically adjust this value to give
    # update with incoming orders. stock value would always be manually updated.    
    store_stock  = schema.Int( title=_(u"Quantity Available For Sale"), description=_(u"Warehouse stock minus pending orders"), default=0)    
    warehouse = schema.Choice( title=_(u"Warehouse"), source=WarehouseSource,  )

class IInventoryModified( IObjectEvent ):
    """
    inventory modified, sent after the modification
    """
    product = schema.Object( interfaces.IShippableContent )
    stock_delta = schema.Int()

class IInventoryAvailabilityModified( IObjectEvent ):
    """
    inventory availability changed by an order
    """
    product = schema.Object( interfaces.IShippableContent )
    order = schema.Object( interfaces.IOrder )
    stock_delta = schema.Int()
    
class IInventoryOrderFulfilled( IObjectEvent ):
    """
    inventory stock changed by order fulfillment
    """
    product = schema.Object( interfaces.IShippableContent )
    order = schema.Object( interfaces.IOrder )
    stock_delta = schema.Int()

class IInventoryBackordered( IObjectEvent ):
    """
    when an item's available inventory ( store_stock ) goes below zero, this
    event is generated.
    """
    product = schema.Object( interfaces.IShippableContent )

class InventoryModified( object ):

    interface.implements( IInventoryModified )
    
    def __init__( self, inventory, product, stock_delta=0 ):
        self.object = inventory
        self.product = product
        self.stock_delta = stock_delta

class InventoryOrderModified( object ):
    
    def __init__( self, inventory, product, order, stock_delta=0 ):
        self.object = inventory
        self.product = product
        self.order = order
        self.stock_delta = stock_delta

class InventoryAvailabilityModified( InventoryOrderModified ):
    
    interface.implements( IInventoryAvailabilityModified )


class InventoryOrderFulfilled( InventoryOrderModified ):
    
    interface.implements( IInventoryOrderFulfilled )

    
class InventoryBackordered( object ):
    
    interface.implements( IInventoryBackordered )
    
    def __init__( self, inventory, product):
        self.object = inventory
        self.product = product

###
# UI Interfaces

class IWarehouseContainerVM( IViewletManager ):
    """ warehouse utility's viewlet manager """

class IWarehouseFulfillmentVM( IViewletManager ):
    """ warehouse fulfilmment viewlet manager """

class IOrderFulfillmentVM( IViewletManager ):
    """ order fulfillment viewlet manager """



"""Base class for integration tests, based on ZopeTestCase and PloneTestCase.

Note that importing this module has various side-effects: it registers a set of
products with Zope, and it sets up a sandbox Plone site with the appropriate
products installed.
"""

from Testing import ZopeTestCase

# Let Zope know about the products we require above-and-beyond a basic
# Plone install (PloneTestCase takes care of these).
from Products.PloneGetPaid.config import PLONE3
if not PLONE3:
    ZopeTestCase.installProduct('CMFonFive')
ZopeTestCase.installProduct('PloneGetPaid')



# Import PloneTestCase - this registers more products with Zope as a side effect
from Products.PloneTestCase.PloneTestCase import PloneTestCase
from Products.PloneTestCase.PloneTestCase import setupPloneSite

from Products.PloneGetPaid.tests.base import PloneGetPaidFunctionalTestCase
from Products.PloneGetPaid.interfaces import IGetPaidManagementOptions
from Products.Five.testbrowser import Browser

# Set up a Plone site.

setupPloneSite()


def baseAfterSetUp( self ):
    """Code that is needed is the afterSetUp of both test cases.
    """

    # This looks like a safe place to install Five.
    ZopeTestCase.installProduct('Five')

    # XXX monkey patch everytime (until we figure out the problem where
    #   monkeypatch gets overwritten somewhere) 
    try:
        from Products.Five import pythonproducts
        pythonproducts.setupPythonProducts(None)
    except ImportError:
        # Not needed in Plone 3
        pass

    # Set up sessioning objects
    ZopeTestCase.utils.setupCoreSessions(self.app)

class WarehouseFunctionalTestCase(PloneGetPaidFunctionalTestCase):
    """Base class for functional integration tests for the warehouse product. 
    This may provide specific set-up and tear-down operations, or provide 
    convenience methods.
    """

    def afterSetUp( self ):
        super(WarehouseFunctionalTestCase,self).afterSetUp()
        self.portal.portal_quickinstaller.installProduct('PloneGetPaid')
        #Here we are setting Documents as shippeable
        options = IGetPaidManagementOptions(self.portal)
        options.shippable_types = ['Document']
        # Set browser and error handling
        self.browser = Browser()
        self.browser.handleErrors = False



"""

Fulfilling an Order.

- fufillment display

  item collection - available items 

    action - create shipment

    - if all items in order, then transition order to delivered
    
    - display form for entry of shipping info

  item collection - back order

  shipment summary listing -


  todo

   - shipping collection

   - on creating shipment, if we have all items, then transition order to delivered

   - if we only have backordered items remaining

   - generate create shipment event

   - add contained to shipment

$Id: $
"""

from copy import copy

import time

from Products.Five.browser import BrowserView
from Products.Five.browser.pagetemplatefile import ZopeTwoPageTemplateFile
from Products.PloneGetPaid.browser import admin_order as order
from Products.PloneGetPaid.browser import widgets
from Products.Five.viewlet import manager, viewlet 

from zope import schema
from zope.formlib import form
from zc.table import column, table
from ore.viewlet import core

from getpaid.core.interfaces import IShippableLineItem, workflow_states, IShippableOrder, IShipment
from getpaid.warehouse.i18n import _
from getpaid.warehouse import interfaces, shipment

from warehouse import manager_template



class WarehouseFulfillment( BrowserView ):
    _download_content = None

    def __call__( self ):
        self.manager = WarehouseFulfillmentVM( self.context, self.request, self )
        self.manager.update()
        if self._download_content is not None:
            self.request.response.setHeader('Content-Type', self._download_content[0] )
            self.request.RESPONSE.setHeader('Content-Disposition','inline;filename=%s-%s.csv' % (self._download_content[2], time.strftime("%Y%m%d",time.localtime())))
            return self._download_content[1]
        return super( WarehouseFulfillment, self).__call__()


class OrderFulfillment( BrowserView ):
    pass

### Viewlets for Warehouse Fulfillment

WarehouseFulfillmentVM = manager.ViewletManager(
    "WarehouseFulfillmentVM",
    interfaces.IWarehouseFulfillmentVM,
    manager_template,
    bases= (order.OrderAdminManagerBase, )
    )

def renderOrderId( order, formatter ):
    return '<a href="@@admin-manage-order/%s/@@fulfillment">%s</a>'%( order.order_id, order.order_id )

class FulfillmentOrderListing( order.OrderListingComponent ):

    columns = copy( order.OrderListingComponent.columns )
    columns.pop(0)
    columns.insert(0, column.GetterColumn( title=_(u"Order Id"), getter=renderOrderId ) )

    
class FulfillmentOrderSearch( order.OrderSearchComponent ):

    form_fields = form.Fields( 
        schema.Choice( **order.define( title=u"Created", __name__=u"creation_date",
                                       values=( [ d[0] for d in order.OrderSearchComponent.date_search_order ] ),
                                       default="last 7 days") ),
        schema.Choice( **order.define( title=u"Status", __name__=u"finance_state",
                                       values= order.OrderSearchComponent._finance_values ) ),
        schema.Choice( **order.define( title=u"Fulfillment",
                                       __name__=u"fulfillment_state",
                                       default="NEW",
                                       values= order.OrderSearchComponent._fulfillment_values ) ),
        schema.TextLine( **order.define( title=u"User Id", __name__=u"user_id") ),
        )
    

### Viewlets for Order Fulfillment

class FulfillmentManagerBase( order.AdminOrderManagerBase ):

    items_by_inventory = None

    def itemsByInventory( self, available=True ):
        # segment items into available, non available sets
        if self.items_by_inventory is not None:
            return self.items_by_inventory.get( available, () )

        items_by_inventory = {}
        
        for i in self.context.shopping_cart.values():
            inventory = getInventory( i )
            if not inventory:
                continue
            key = ( inventory.stock >= i.quantity )
            items_by_inventory.setdefault( key, [] ).append( i )

        self.items_by_inventory = items_by_inventory
        return items_by_inventory.get( available, () )
        
OrderFulfillmentVM = manager.ViewletManager(
    "OrderFulfillmentVM",
    interfaces.IOrderFulfillmentVM,
    manager_template,
    bases= ( FulfillmentManagerBase, )
    )    

def getInventory( item ):
    if not IShippableLineItem.providedBy( item ):
        return None
    
    payable = item.resolve()
    if payable is None:
        return None

    return interfaces.IProductInventory( payable )

def renderPickBin( item, formatter ):
    if formatter.inventory is None:
        return u"N/A"
    return formatter.inventory.pickbin

def renderPallet( item, formatter ):
    inventory= getInventory( item )
    if inventory is None:
        formatter.inventory = None
        return u'N/A'
    formatter.inventory = inventory
    return inventory.pallet

class BackorderList( order.OrderContentsComponent ):

    columns = copy( order.OrderContentsComponent.columns )
    columns.pop(5) # total
    columns.pop(3) # price
    columns.pop(0) # selection
    
    collection_name = _(u"Backordered Items")
    
    def show( self, **kw ):
        self.line_items = self.manager.itemsByInventory( available=False )
        if len( self.line_items ) == 0:
            return False
        return True

    def update( self ):
        self.actions = ()
        return super( order.OrderContentsComponent, self).update()

class ShippedItems( order.OrderContentsComponent ):

    columns = copy( order.OrderContentsComponent.columns )
    columns.pop(5) # total
    columns.pop(3) # price
    columns.pop(0) # selection
    
    collection_name = _(u"Shipped Items")
    
    def show( self, inline=False, **kw ):
        if not inline:
            return True        
        if len( self.line_items ) == 0:
            return False
        return True

    def render( self ):
        if self.show( inline=True ):
            return super( ShippedItems, self).render()
        return ''
    
    def update( self ):
        self.actions = ()
        self.line_items = self.manager.itemsByStates(
            (workflow_states.item.SHIPPED,),
            )
        return super( order.OrderContentsComponent, self).update()    
        
class OrderPickList( order.OrderContentsComponent ):

    columns = copy( order.OrderContentsComponent.columns )
    columns.pop(5) # total
    columns.pop(3) # price
    columns.append( 
        column.GetterColumn( title=_(u"Pallet"), getter=renderPallet)
        )
    columns.append( 
        column.GetterColumn( title=_(u"Pickbin"), getter=renderPickBin)
        )

    actions = None  # reset standard container actions
    collection_name = _(u"Order Items") # fieldset label

    template = ZopeTwoPageTemplateFile('picklist.pt')
    
    form_fields = form.Fields( IShipment )
    form_fields = form_fields.omit('creation_date', 'service_code', 'service')
    
    #Very important not to forget this, otherwise we have only on decimal place on price
    form_fields['shipping_cost'].custom_widget = widgets.PriceWidget
    
    form_name = _(u"Create Shipment") # inner fieldset form label
    form_description = _(u"Select Items to Ship, and Enter Shipping Details")
    
    def show( self, inline=False, **kw ):
        if not inline:
            return True
        if len(self.line_items):
            return True
        if self.status:
            return True
        return False

    def render( self ):
        if self.show( inline=True ):
            return super( OrderPickList, self).render()
        return ''

    def filteredItems( self ):
        # we want only items in inventory that aren't shipped
        i_items =  self.manager.itemsByInventory( )
        s_items =  self.manager.itemsByStates( (workflow_states.item.NEW,) )
        filtered = []
        for i in s_items:
            if not i in i_items:
                continue
            filtered.append( i )
        return filtered
    
    def update( self ):
        # pop the selection if the order is already fulfilled
        if self.context.fulfillment_state == workflow_states.order.fulfillment.DELIVERED:
            columns = copy( self.columns )
            columns.pop(0)
            self.columns = columns
        self.line_items = self.filteredItems()
        return super( order.OrderContentsComponent, self).update()

    def condition_create_shipment( self, action ):
        if not IShippableOrder.providedBy( self.context._object ):
            return False
        if self.context.fulfillment_state == workflow_states.order.fulfillment.DELIVERED:
            return False
        if not self.line_items:
            return False
        return True
        
    @form.action(_(u"Create Shipment"), condition="condition_create_shipment" )
    def handle_create_shipment( self, action, data ):
        items = self.selection_column.getSelected( self.line_items, self.request )
        if not items:
            self.status = _(u"Please Select Items to Ship")
            return
        
        o_shipment = shipment.Shipment()
        
        order = self.context._object
        # this gets attached normally via the warehouse
        # event subscriber, we do it lazy here for existing orders
        if getattr( order, 'shipments', None) is None:
            order.shipments = shipment.OrderShipments()

        # modify shipment with supplied form fields
        adapters = { IShipment : o_shipment }
        form.applyChanges( None, self.form_fields, data, adapters )

        # store shipment
        sid = str( len( order.shipments ) + 1 )
        order.shipments[ sid ] = o_shipment

        # populate shipment with items
        for i in items:
            o_shipment[ i.item_id ] = i.clone()
            i.fulfillment_workflow.fireTransition('ship')

        # attempt to transition order
        self.transitionOrder()

        # clear manager item cache and fetch
        self.status = _(u"Shipment Created")
        self.update()

    def transitionOrder( self ):
        
        # clear manager item caches
        self.manager.items_by_state = None
        self.manager.items_by_inventory = None
        
        # if we have no more items         
        items = self.manager.itemsByStates( ( workflow_states.item.NEW, ) )
        if not len(items):
            if self.context.fulfillment_state == workflow_states.order.fulfillment.NEW:
                self.context.fulfillment_workflow.fireTransition('process-order')
                
            if self.context.fulfillment_state == workflow_states.order.fulfillment.PROCESSING:
                self.context.fulfillment_workflow.fireTransition('deliver-processing-order')                
        
def g_created( item, formatter ):
    return item.creation_date.isoformat()

def g_item( item, formatter ):
    return len(item)

def g_track( item, formatter ):
    return getattr( item, 'tracking_code', _(u'n/a'))

class OrderShipments( viewlet.ViewletBase ):

    __allow_access_to_unprotected_subobjects__ = 1
    
    template = ZopeTwoPageTemplateFile('item-collection.pt')
    
    columns = [
        column.GetterColumn( title=_(u"Created"), getter=g_created),
        column.GetterColumn( title=_(u"Item Count"), getter=g_item),
        column.GetterColumn( title=_(u"Tracking"), getter=g_track),
        ]
    
    collection_name = _(u"Shipments")
    order = 20

    def show( self, inline=False, **kw ):
        if not inline:
            return True
        if not IShippableOrder.providedBy( self.context._object ):
            return False
        shipments = getattr( self.context, 'shipments', ())
        if not len(shipments):
            return False
        return True
        
    def render( self ):
        if self.show( inline=True):
            return self.__of__( self.__parent__).template()
        return ''

    def listing( self ):
        values = self.context.shipments.values()
        formatter = table.StandaloneFullFormatter(
            self.context,
            self.request,
            values,
            prefix="ship",
            visible_column_names = [c.name for c in self.columns ],
            columns = self.columns
            )
        formatter.cssClasses['table'] = 'listing'
        return formatter()
    
class OrderSummary( order.OrderSummaryComponent ):
    """ to be modified for sanity """

##    template = ZopeTwoPageTemplateFile('order-summary.pt')

##     def getShippingAddress( self ):
        
##         if not self.order.shipping_address.ship_same_billing:
##             return super( FulfillmentOrderSummary, self).getShippingAddress()

##         address = self.getBillingAddress()
##         for k in address.keys():
##             address[ k.replace('bill', 'ship') ] = address[k]
##         return address
            



"""
$Id: $
"""

from zope.app.container.ordered import OrderedContainer
from zope.interface import implements
from zope.schema.fieldproperty import FieldProperty
from datetime import datetime
from persistent import Persistent

from getpaid.core import interfaces

class OrderShipments( OrderedContainer ):

    implements( interfaces.IShipmentContainer )

class Shipment( OrderedContainer ):

    implements( interfaces.IShipment )

    tracking_code = FieldProperty( interfaces.IShipment['tracking_code'] )
    service_code = FieldProperty( interfaces.IShipment['service_code'] )
    service_name = FieldProperty( interfaces.IShipment['service'] )
    shipping_cost = FieldProperty( interfaces.IShipment['shipping_cost'] )

    creation_date = FieldProperty( interfaces.IShipment['creation_date'] )

    def __init__( self ):
        self.creation_date = datetime.now()
        super( Shipment, self).__init__()
    
    


    




# geocoding


from zope import interface, component

from getpaid.core.interfaces import IStoreSettings, IShippableLineItem, IOrder, IOriginRouter
from getpaid.core.payment import ContactInformation

import interfaces

class OriginRouter( object ):
    
    " warehouse aware origin router "
    
    component.adapts( IOrder )
    
    interface.implements( IOriginRouter )
    
    def __init__( self, context ):
        self.context = context

    def getOrigin( self ):
        warehouses = self.getOrderWarehouses()
        if not warehouses:
            fallback_router = component.getAdapter( self.context, IOriginRouter, name="default" )
            return fallback_router.getOrigin()
        assert len(warehouses) == 1, "only one origin warehouse allowed per order, atm"
        warehouse = warehouses.pop()
        contact = self.getStoreContact()
        contact.name = warehouse.name
        return contact, warehouse.location
        
    def getOrderWarehouses( self ):
        items = filter( IShippableLineItem.providedBy, self.context.shopping_cart.values())
        if not items:
            return
            
        warehouse_names = set()
        for i in items:
            shippable = i.resolve()
            if shippable is None:
                continue
            inventory = interfaces.IProductInventory( shippable )
            warehouse_names.add( inventory.warehouse )
        
        container = component.getUtility( interfaces.IWarehouseContainer )
        return filter( None, map( container.get, warehouse_names ) )
        
    def getStoreContact( self ):
        store_settings = component.getUtility( IStoreSettings )
        contact = ContactInformation( name = ( store_settings.contact_company or store_settings.store_name ),
                                      phone_number = store_settings.contact_phone,
                                      email = store_settings.contact_email )
        return contact



from Products.Five.viewlet import viewlet
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from getpaid.core import interfaces

class WarehouseInfo( viewlet.ViewletBase ):
    
    template = ViewPageTemplateFile('order-warehouse.pt')
    
    order = 3
    
    def update( self ):
        router = interfaces.IOriginRouter( self.__parent__.context )
        self.contact, self.location = router.getOrigin()

    def render( self ):
        return self.__of__( self.__parent__ ).template()
        
    def show( self, **kw ):
        # for viewlet filtering by order state (finance, fullfillment kw args)
        return True


"""
$Id: $



these views are directly ontop of the warehouse utility and its data, 
and for the sake of z2/plone.. manage context as needed via form 
variables. 

"""

import os

from getpaid.warehouse import interfaces, warehouse
from getpaid.warehouse.interfaces import _
from getpaid.core.interfaces import IAddress

from zope import component
from zope.formlib import form

from zc.table import column
from ore.viewlet.container import ContainerViewlet
from zExceptions import Redirect
from Products.Five.viewlet import manager
from Products.Five.browser import BrowserView
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from Products.PloneGetPaid.browser import base
from Products.PloneGetPaid.browser.widgets import CountrySelectionWidget, StateSelectionWidget
from Products.CMFPlone.utils import safe_unicode

from getpaid.core.payment import Address

class Container( BrowserView ):
    """ admin view on warehouse utility """
    

def warehouseURL( item, formatter ):
    return '<a href="@@pgp-view-warehouse?wid=%s">%s</a>'  % (item.__name__, safe_unicode(item.name))

    
class Listing( ContainerViewlet ):
    """ viewlet for managing contents """
    
    actions = ContainerViewlet.actions.copy()
    template = ViewPageTemplateFile('warehouse-actions.pt')
    
    columns = [
        column.SelectionColumn( lambda item: item.name, name="selection"),
        column.GetterColumn( title=_(u"Name"), getter=warehouseURL )
       ]

    selection_column = columns[0]
        
    def getContainerContext( self ):
        return component.getUtility( interfaces.IWarehouseContainer )
    
    @form.action( _(u"Add") )
    def handle_add( self, action, data):
        self.request.response.redirect('@@pgp-add-warehouse')
        return
        
_prefix = os.path.dirname( base.__file__ )

manager_template = os.path.join( _prefix, "templates", "viewlet-manager.pt")
admin_template = os.path.join( _prefix, "templates", 'settings-page.pt')

ContainerVM = manager.ViewletManager(
                "ContainerVM", 
                interfaces.IWarehouseContainerVM,
                manager_template,
                )

class Add( base.BaseFormView ):
    """ add a warehouse """
    
    template = ViewPageTemplateFile( admin_template )
    
    form_fields = form.Fields( interfaces.IWarehouse )
    form_fields = form_fields.select('name') + form.Fields( interfaces.IAddress )
    form_fields['country'].custom_widget = CountrySelectionWidget
    form_fields['state'].custom_widget = StateSelectionWidget


    def setUpWidgets(self, ignore_request=False):
        self.adapters = {}
        self.widgets = form.setUpDataWidgets(
            self.form_fields, self.prefix, self.context, self.request,
            ignore_request=ignore_request
            )

    @form.action( _(u"Add Warehouse"), condition=form.haveInputWidgets )
    def handle_add_warehouse( self, action, data ):
        adapters = { interfaces.IWarehouse : warehouse.Warehouse(),
                     interfaces.IAddress   : Address() }
                     
        form.applyChanges( self.context, self.form_fields, data, adapters )
        
        container = component.getUtility( interfaces.IWarehouseContainer )
        w = adapters[ interfaces.IWarehouse ]
        w.location = adapters[ interfaces.IAddress ]
        container[ w.name ] = w
        self.request.response.redirect('@@pgp-view-warehouse?wid=%s'% w.name )

class Edit( base.BaseFormView ):
    """ edit a warehouse """
    
    template = ViewPageTemplateFile( admin_template )
    
    form_fields = form.Fields( interfaces.IWarehouse )
    form_fields['name'].for_display = True
    
    form_fields = form.Fields( form_fields['name'], form.Fields( interfaces.IAddress ) )
    form_fields['country'].custom_widget = CountrySelectionWidget
    form_fields['state'].custom_widget = StateSelectionWidget
        
    
    form_name = _(u'Warehouse Information')
                
    def setUpWidgets( self, ignore_request=False ):
        self.adapters = self.adapters or {}
        self.widgets = form.setUpEditWidgets(
            self.form_fields, self.prefix, self.context, self.request,
            adapters=self.adapters, 
            ignore_request=ignore_request
            )
            
    def update( self ):
        self.form_fields['name'].for_display = True
        wid = self.request.get('wid', None)
        container = component.getUtility( interfaces.IWarehouseContainer )
        if not wid or not wid in container:
            raise Redirect('@@pgp-warehouse-admin')
        w = container[ wid ]
        self.adapters = { interfaces.IWarehouse : w,
                          interfaces.IAddress   : w.location }
        self.hidden_form_vars = {'wid':wid}                          
        super( Edit, self).update()

class View( base.BaseFormView ):
    """ view a warehouse """
    
    template = ViewPageTemplateFile( admin_template )
    form_fields = form.Fields( interfaces.IWarehouse )
    form_fields = form.Fields( form_fields.select('name') + form.Fields( interfaces.IAddress ) )
        
    def setUpWidgets( self, ignore_request=False ):
        self.adapters = self.adapters or {}
        self.widgets = form.setUpEditWidgets(
            self.form_fields, self.prefix, self.context, self.request,
            adapters=self.adapters, for_display=True,
            ignore_request=ignore_request
            )

    def update( self ):
        wid = self.request.get('wid', None)
        container = component.getUtility( interfaces.IWarehouseContainer )
        if not wid or not wid in container:
            self.request.response.redirect('@@pgp-warehouse-admin')
        w = container[ wid ]
        self.adapters = { interfaces.IWarehouse : w,
                          interfaces.IAddress   : w.location }
        self.hidden_form_vars = {'wid':wid}
        super( View, self).update()
        self.form_name = \
                u'Warehouse Information (<a href="@@pgp-edit-warehouse?wid=%s">Edit</a>)'%wid

    def isOrdered( self ):
        return False




from getpaid.core import options
from getpaid.core import interfaces as icore
from zope.event import notify

import interfaces, shipment

Inventory = options.PersistentOptions.wire( 
                "Inventory", 
                "getpaid.content.inventory",
                interfaces.IProductInventory 
                )

def handleNewOrder( order, event ):
    """
    keep track of the amount product we have on hand *MINUS* our 
    outstanding orders. ie. what product do we have to sell, 
    including our unfufilled orders.
    
    what constitutes a new order, an order which is the chargeable 
    finance state.

    attaches shipments to a new shippable order
    """
    
    if event.destination != icore.workflow_states.order.finance.CHARGEABLE:
        return

    if not event.source in ( icore.workflow_states.order.finance.REVIEWING, ):
        return 

    if icore.IShippableOrder.providedBy( order ):
        order.shipments = shipment.OrderShipments()
            
    for item in order.shopping_cart.values():
        if not ( icore.IShippableLineItem.providedBy( item ) 
                 and icore.IPayableLineItem.providedBy( item ) ):
            continue

        payable = item.resolve()
        if payable is None:
            continue
        
        inventory = interfaces.IProductInventory( payable )
        inventory.store_stock -= item.quantity
        
        if inventory.store_stock < 0 and (inventory.store_stock + item.quantity) >= 0:
            notify( interfaces.InventoryBackordered( inventory, payable ) )

        notify(
            interfaces.InventoryAvailabilityModified( inventory, payable, order, item.quantity )
            )

def handleFufilledOrder( order, event ):
    """
    when we fufill an order we decrement, our on hand stock, to come 
    inline with our fufillment of this order.
    """ 
    if not event.destination in ( 
        icore.workflow_states.order.fulfillment.DELIVERED,
        icore.workflow_states.shippable_order.fulfillment.DELIVERED,
        icore.workflow_states.order.fulfillment.WILL_NOT_DELIVER,
        icore.workflow_states.shippable_order.fulfillment.WILL_NOT_DELIVER):
        return
            
    for item in order.shopping_cart.values():
        if not ( icore.IShippableLineItem.providedBy( item ) 
                 and icore.IPayableLineItem.providedBy( item ) ):
            continue
            
        payable = item.resolve() 
        if payable is None:
            continue
            
        inventory = interfaces.IProductInventory( payable )
        inventory.stock -= item.quantity
        

        notify(
            interfaces.InventoryOrderFulfilled( inventory, payable, order, item.quantity )
            )
            
        



"""
$Id: $
"""

from zope import component, interface
from getpaid.core.interfaces import IPluginManager
from getpaid.warehouse import interfaces 
from getpaid.warehouse import warehouse

class WarehousePlugin( object ):

    interface.implements( IPluginManager )
    
    title = "Warehouse/Inventory Management"
    description = ""
    
    def __init__(self, context ):
        self.context = context
        
    def install( self ):
        sm = self.context.getSiteManager()
        
        utility = sm.queryUtility( interfaces.IWarehouseContainer )
        
        if utility is not None:
            return

        warehouses = warehouse.WarehouseContainer()
         
        try:
            sm.registerUtility(component=warehouses, provided=interfaces.IWarehouseContainer )
        except TypeError:
            # BBB for Zope 2.9
            sm.registerUtility(interface=interfaces.IWarehouseContainer, utility=warehouses)
        
    def uninstall( self ):
        pass
        
    def status( self ):
        return component.queryUtility( interfaces.WarehouseContainer ) is not None
        
def storeInstalled( object, event ):
    return WarehousePlugin( object ).install()


"""
$Id: $
"""

from persistent import Persistent

from zope import interface
from zope.app.container.btree import BTreeContainer
from zope.app.container.contained import Contained

import interfaces

class WarehouseContainer( BTreeContainer ):
    
    interface.implements( interfaces.IWarehouseContainer )
    
class Warehouse( Persistent, Contained ):

    interface.implements( interfaces.IWarehouse )

    name = ""



"""
$Id: $
"""

from Products.Five.formlib import formbase    
from Products.PloneGetPaid.browser.content import PayableFormView

from zope.formlib import form
from zope.event import notify
from getpaid.warehouse import interfaces
from getpaid.warehouse.interfaces import _

class ContentInventory( PayableFormView, formbase.EditForm ):
    
    form_fields = form.Fields( interfaces.IProductInventory )
    form_fields['store_stock'].for_display = True
    
    form_name = "Product Inventory"
    allowed = True
    
    @form.action(_("Apply"), condition=form.haveInputWidgets)
    def handle_edit_action(self, action, data):
        
        inventory = self.adapters[ interfaces.IProductInventory ]

        _modified = False
        if inventory.stock != data.get('stock', inventory.stock):
            # if we're adding to the current stock on hand we should
            # also increment the amount that can be ordered (includes unfufilled
            # orders ). we use reset store stock to the stock minus the
            # current delta between stock/storestock
            delta = inventory.stock - inventory.store_stock
            inventory.store_stock = data.get('stock') - delta

            # notify on change in stock levels 
            _modified = True            
            stock_delta = data.get('stock') - inventory.stock
            
        retvalue = super(ContentInventory, self).handle_edit_action.success_handler( self, action, data )

        if _modified:
            notify(
                interfaces.InventoryModified( inventory, self.context, stock_delta )
                )
            
        return retvalue
            



from setuptools import setup, find_packages

setup(
    name="getpaid.warehouse",
    version="0.2",
    author="Getpaid Community",
    author_email="getpaid-dev@googlegroups.com",
    description="Warehouse module for ecommerce framework",
    url="http://code.google.com/p/getpaid",
    classifiers = [
        "Framework :: Zope2",
        "Framework :: Zope3",
        "Framework :: Plone"
        ],
    packages=find_packages('src'),
    package_dir={'':'src'},
    namespace_packages=['getpaid'],
    include_package_data=True,
    install_requires = [ 'setuptools',
                         'getpaid.core',
                         ],
    zip_safe = False,
    )



"""PloneGetPaid functional doctests.  This module collects all *.txt
files in the tests directory and runs them. (stolen from Plone)
"""

import os, sys

import glob
import doctest
import unittest
from Globals import package_home
from Products.PloneTestCase import PloneTestCase
from Testing.ZopeTestCase import FunctionalDocFileSuite as Suite

from getpaid.warehouse.config import GLOBALS

# Load products
from getpaid.warehouse.tests.base import WarehouseFunctionalTestCase

REQUIRE_TESTBROWSER = []

OPTIONFLAGS = (doctest.REPORT_ONLY_FIRST_FAILURE |
               doctest.ELLIPSIS |
               doctest.NORMALIZE_WHITESPACE)

def list_doctests():
    home = package_home(GLOBALS)
    return [filename for filename in
            glob.glob(os.path.sep.join([home, 'tests', '*.txt']))]

def list_nontestbrowser_tests():
    return [filename for filename in list_doctests()
            if os.path.basename(filename) not in REQUIRE_TESTBROWSER]

def test_suite():

    # BBB: We can obviously remove this when testbrowser is Plone
    #      mainstream, read: with Five 1.4.
    try:
        import Products.Five.testbrowser
    except ImportError:
        print >> sys.stderr, ("WARNING: testbrowser not found - you probably"
                              "need to add Five 1.4 to the Products folder. "
                              "testbrowser tests skipped")
        filenames = list_nontestbrowser_tests()
        print 1
    else:
        filenames = list_doctests()
        print 2
    print filenames


    return unittest.TestSuite(
        [Suite(os.path.basename(filename),
               optionflags=OPTIONFLAGS,
               package='getpaid.warehouse.tests',
               test_class=WarehouseFunctionalTestCase)
         for filename in filenames]
        )
